@@ -35,6 +35,10 @@ struct Q_QUICK3DRUNTIMERENDER_EXPORT QSSGRenderItem2D : public QSSGRenderNode
 
     QSSGRenderItem2D();
     ~QSSGRenderItem2D() override;
+
+    QSSGRenderItem2DHandle ih;
+
+    QPointer<QObject> m_frontEndObject;
 };
 QT_END_NAMESPACE
 
