@@ -24,6 +24,8 @@
 #include <QtCore/qoperatingsystemversion.h>
 #include <QtCore/qloggingcategory.h>
 
+#include <TargetConditionals.h>
+
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(lcQuick3DXr);
@@ -45,6 +47,7 @@ public:
     ~CompositorLayer() final
     {
         m_xrManager = nullptr;
+        [m_layerRenderer release];
         m_layerRenderer = nullptr;
         m_worldTrackingProvider = nullptr;
         m_arSession = nullptr;
@@ -119,6 +122,10 @@ public:
 
         if (m_layerRenderer) {
             QMutexLocker locker(&m_mutex);
+            // NOTE: The layer renderer is our handle to the compositor layer, so we need to ensure that
+            //       we need to ensure that it stays alive long enough for us to know if we should tear down
+            //       the compositor layer or not.
+            [m_layerRenderer retain];
             checkRenderState();
             emit layerRendererReady();
         }
@@ -173,6 +180,10 @@ public:
 
         if (m_arSession) {
             qCDebug(lcQuick3DXr, "Stopping AR session");
+            // NOTE: We're stopping the AR session, so we need to ensure that the AR session
+            //       doesn't get posted at some later point, so we change the dispatch queue to
+            //       use the main queue (which is the GUI thread) before calling stop.
+            ar_session_set_data_provider_state_change_handler_f(m_arSession, dispatch_get_main_queue(), this, &onArStateChanged);
             ar_session_stop(m_arSession);
             m_arSession = nullptr;
             m_arTrackingState = QQuick3DXrManagerPrivate::ArTrackingState::Uninitialized;
@@ -187,7 +198,7 @@ public:
 
     Q_INVOKABLE static void destroy(QQuick3DXrManagerPrivate *mngr, QQuickWindow *window, CompositorLayer *compositorLayer)
     {
-        QSSG_ASSERT(window != nullptr && mngr != nullptr, return);
+        QSSG_ASSERT(mngr != nullptr, return);
 
         qCDebug(lcQuick3DXr) << "Destroying compositor layer";
 
@@ -201,11 +212,13 @@ public:
         // thread and resources created on it. Instead we just invalidate the render
         // control and cleanup the nodes on the render thread. This is not ideal,
         // but it's what we have for now...
-        auto *d = QQuickWindowPrivate::get(window);
-        d->cleanupNodesOnShutdown();
-        mngr->releaseResources();
-        if (auto *rc = d->renderControl)
-            rc->invalidate();
+        if (window) { // If we never got past the setup we don't have a window.
+            auto *d = QQuickWindowPrivate::get(window);
+            d->cleanupNodesOnShutdown();
+            mngr->releaseResources();
+            if (auto *rc = d->renderControl)
+                rc->invalidate();
+        }
 
         // We're holding onto the lock, so defer the delete until we're done here.
         // All reasources should be released at this point anyways.
@@ -283,6 +296,9 @@ private:
 
         QMutexLocker lock(&that->m_arSessionMtx);
 
+        // If the ar session is null we're called after we tore down the session.
+        QSSG_ASSERT_X(that->m_arSession != nullptr, "AR session is not running, skipping state change!", return);
+
         const auto oldState = that->m_arTrackingState;
         switch (new_state) {
         case ar_data_provider_state_initialized:
@@ -547,8 +563,6 @@ bool QQuick3DXrManagerPrivate::initialize()
     if (!m_compositorLayer->isInitialized()) {
         if (auto *visionOSApplicaton = qGuiApp->nativeInterface<QNativeInterface::QVisionOSApplication>()) {
             visionOSApplicaton->setImmersiveSpaceCompositorLayer(m_compositorLayer);
-            m_compositorLayer->init(this);
-
             // FIXME: We don't actually handle the case where the rendere changes or we get multiple calls should do something.
             QObject::connect(m_compositorLayer, &CompositorLayer::layerRendererReady, q, &QQuick3DXrManager::initialized, Qt::ConnectionType(Qt::SingleShotConnection | Qt::QueuedConnection));
             QObject::connect(m_compositorLayer, &CompositorLayer::renderStateChanged, q, [q](QQuick3DXrManagerPrivate::RenderState state) {
@@ -599,6 +613,8 @@ bool QQuick3DXrManagerPrivate::initialize()
                         break;
                 }
             }, Qt::DirectConnection);
+
+            m_compositorLayer->init(this);
         }
         return false;
     }
@@ -708,16 +724,18 @@ void QQuick3DXrManagerPrivate::teardown()
         m_compositorLayer = nullptr;
     }
 
-    m_renderThread->quit();
-    m_renderThread->wait();
-    delete m_renderThread;
-    m_renderThread = nullptr;
+    if (m_renderThread) {
+        m_renderThread->quit();
+        m_renderThread->wait();
+        delete m_renderThread;
+        m_renderThread = nullptr;
+    }
 }
 
 void QQuick3DXrManagerPrivate::setMultiViewRenderingEnabled(bool enable)
 {
     Q_UNUSED(enable);
-    qWarning() << "Changing multiview rendering is not supported at runtime on VisionOS!";
+    qCWarning(lcQuick3DXr) << "Changing multiview rendering is not supported at runtime on visionOS!";
 }
 
 bool QQuick3DXrManagerPrivate::isMultiViewRenderingEnabled() const
@@ -725,10 +743,11 @@ bool QQuick3DXrManagerPrivate::isMultiViewRenderingEnabled() const
     return m_multiviewRenderingEnabled;
 }
 
-void QQuick3DXrManagerPrivate::setPassthroughEnabled(bool enable)
+bool QQuick3DXrManagerPrivate::setPassthroughEnabled(bool enable)
 {
     Q_UNUSED(enable);
-    Q_UNIMPLEMENTED(); qWarning() << Q_FUNC_INFO;
+    qCWarning(lcQuick3DXr) << "Changing passthrough is not supported at runtime on visionOS!";
+    return supportsPassthrough();
 }
 
 QtQuick3DXr::ReferenceSpace QQuick3DXrManagerPrivate::getReferenceSpace() const
@@ -739,16 +758,14 @@ QtQuick3DXr::ReferenceSpace QQuick3DXrManagerPrivate::getReferenceSpace() const
 
 void QQuick3DXrManagerPrivate::setReferenceSpace(QtQuick3DXr::ReferenceSpace newReferenceSpace)
 {
-    // FIXME: Not sure if it's possible to set a reference space on VisionOS
     Q_UNUSED(newReferenceSpace);
-    Q_UNIMPLEMENTED(); qWarning() << Q_FUNC_INFO;
+    qCWarning(lcQuick3DXr) << "Changing reference space is not supported at runtime on visionOS!";
 }
 
 void QQuick3DXrManagerPrivate::setDepthSubmissionEnabled(bool enable)
 {
-    Q_UNUSED(enable);
     if (!enable)
-        qWarning("Depth submission is required on VisionOS");
+        qCWarning(lcQuick3DXr, "Depth submission is required on visionOS");
 }
 
 void QQuick3DXrManagerPrivate::update()
@@ -817,7 +834,7 @@ void QQuick3DXrManagerPrivate::initInputManager(QQuick3DXrInputManager *im)
 void QQuick3DXrManagerPrivate::setSamples(int samples)
 {
     Q_UNUSED(samples);
-    qWarning("Setting samples is not supported");
+    qCWarning(lcQuick3DXr) << "Changing sample count is not supported on visionOS!";
 }
 
 QString QQuick3DXrManagerPrivate::runtimeName() const
@@ -866,7 +883,7 @@ void QQuick3DXrManagerPrivate::setupShadingRateMap(QQuickWindow *window, QRhiSha
 void QQuick3DXrManagerPrivate::releaseResources()
 {
     qCDebug(lcQuick3DXr) << "Releasing resources";
-    Q_ASSERT(QThread::currentThread() == m_renderThread);
+    Q_ASSERT((m_renderThread == nullptr) || QThread::currentThread() == m_renderThread);
     delete m_srmRenderPassDesc;
     m_srmRenderPassDesc = nullptr;
     delete m_rhiDepthTexture;
@@ -1099,6 +1116,15 @@ bool QQuick3DXrManagerPrivate::renderFrameImpl(QMutexLocker<QMutex> &locker, QWa
             renderControl->render();
             renderControl->endFrame();
         }
+
+#if TARGET_OS_SIMULATOR == 1
+        // With multiview this indicates that the frame with both eyes is ready from
+        // the 3D APIs perspective. Without multiview this is done - and so the
+        // signal is emitted - multiple times (twice) per "frame" (eye).
+        QRhiRenderTarget *rt = QQuickWindowPrivate::get(window)->activeCustomRhiRenderTarget();
+        if (rt->resourceType() == QRhiResource::TextureRenderTarget && static_cast<QRhiTextureRenderTarget *>(rt)->description().colorAttachmentAt(0)->texture())
+            emit q->frameReady();
+#endif // TARGET_OS_SIMULATOR
     }
 
     id<MTLCommandBuffer> commandBuffer = [static_cast<const QRhiMetalNativeHandles*>(renderControl->rhi()->nativeHandles())->cmdQueue commandBuffer];
